@@ -23,10 +23,13 @@
 #
 ############################################################################
 
-from dumper import *
+from utils import DisplayFormat
+from dumper import Children, SubItem
+
 
 def qform__std__array():
-    return arrayForms()
+    return [DisplayFormat.ArrayPlot]
+
 
 def qdump__std__array(d, value):
     size = value.type[1]
@@ -36,7 +39,8 @@ def qdump__std__array(d, value):
 
 
 def qform__std____1__array():
-    return arrayForms()
+    return [DisplayFormat.ArrayPlot]
+
 
 def qdump__std____1__array(d, value):
     qdump__std__array(d, value)
@@ -59,24 +63,39 @@ def qdump__std__complex(d, value):
     innerType = value.type[0]
     (real, imag) = value.split('{%s}{%s}' % (innerType.name, innerType.name))
     d.putValue("(%s, %s)" % (real.display(), imag.display()))
-    d.putNumChild(2)
+    d.putExpandable()
     if d.isExpanded():
         with Children(d, 2, childType=innerType):
             d.putSubItem("real", real)
             d.putSubItem("imag", imag)
 
+
 def qdump__std____1__complex(d, value):
     qdump__std__complex(d, value)
 
 
 def qdump__std__deque(d, value):
     if d.isQnxTarget():
-        qdump__std__deque__QNX(d, value)
-        return
-    if d.isMsvcTarget():
-        qdump__std__deque__MSVC(d, value)
-        return
+        qdumpHelper__std__deque__qnx(d, value)
+    elif d.isMsvcTarget():
+        qdumpHelper__std__deque__msvc(d, value)
+    elif value.hasMember("_M_impl"):
+        qdumpHelper__std__deque__libstdcxx(d, value)
+    elif value.hasMember("__start_"):
+        qdumpHelper__std__deque__libcxx(d, value)
+    elif value.type.size() == 10 * d.ptrSize():
+        qdumpHelper__std__deque__libstdcxx(d, value)
+    elif value.type.size() == 6 * d.ptrSize():
+        qdumpHelper__std__deque__libcxx(d, value)
+    else:
+        qdumpHelper__std__deque__libstdcxx(d, value)
 
+
+def qdump__std____1__deque(d, value):
+    qdumpHelper__std__deque__libcxx(d, value)
+
+
+def qdumpHelper__std__deque__libstdcxx(d, value):
     innerType = value.type[0]
     innerSize = innerType.size()
     bufsize = 1
@@ -84,7 +103,7 @@ def qdump__std__deque(d, value):
         bufsize = 512 // innerSize
 
     (mapptr, mapsize, startCur, startFirst, startLast, startNode,
-              finishCur, finishFirst, finishLast, finishNode) = value.split("pppppppppp")
+     finishCur, finishFirst, finishLast, finishNode) = value.split("pppppppppp")
 
     size = bufsize * ((finishNode - startNode) // d.ptrSize() - 1)
     size += (finishCur - finishFirst) // innerSize
@@ -107,7 +126,8 @@ def qdump__std__deque(d, value):
                     pcur = pfirst
                     pnode = newnode
 
-def qdump__std____1__deque(d, value):
+
+def qdumpHelper__std__deque__libcxx(d, value):
     mptr, mfirst, mbegin, mend, start, size = value.split("pppptt")
     d.check(0 <= size and size <= 1000 * 1000 * 1000)
     d.putItemCount(size)
@@ -122,7 +142,8 @@ def qdump__std____1__deque(d, value):
                 base = d.extractPointer(mfirst + k * ptrSize)
                 d.putSubItem(i, d.createValue(base + j * innerSize, innerType))
 
-def qdump__std__deque__QNX(d, value):
+
+def qdumpHelper__std__deque__qnx(d, value):
     innerType = value.type[0]
     innerSize = innerType.size()
     if innerSize <= 1:
@@ -156,9 +177,10 @@ def qdump__std__deque__QNX(d, value):
                 if mapsize <= block:
                     block -= mapsize
                 d.putSubItem(i, map[block][offset])
-                myoff += 1;
+                myoff += 1
 
-def qdump__std__deque__MSVC(d, value):
+
+def qdumpHelper__std__deque__msvc(d, value):
     innerType = value.type[0]
     innerSize = innerType.size()
     if innerSize <= 1:
@@ -186,6 +208,7 @@ def qdump__std__deque__MSVC(d, value):
                 d.putSubItem(i, d.createValue(address, innerType))
                 myoff += 1
 
+
 def qdump__std____debug__deque(d, value):
     qdump__std__deque(d, value)
 
@@ -217,8 +240,13 @@ def qdump__std__list(d, value):
                 d.putSubItem(i, d.createValue(p + 2 * d.ptrSize(), innerType))
                 p = d.extractPointer(p)
 
+
 def qdump__std__list__QNX(d, value):
-    (proxy, head, size) = value.split("ppp")
+    try:
+        _ = value["_Mypair"]["_Myval2"]["_Myproxy"]
+        (proxy, head, size) = value.split("ppp")
+    except Exception:
+        (head, size) = value.split("pp")
     d.putItemCount(size, 1000)
 
     if d.isExpanded():
@@ -229,12 +257,15 @@ def qdump__std__list__QNX(d, value):
                 d.putSubItem(i, d.createValue(p + 2 * d.ptrSize(), innerType))
                 p = d.extractPointer(p)
 
+
 def qdump__std____debug__list(d, value):
     qdump__std__list(d, value)
 
+
 def qdump__std____cxx11__list(d, value):
     qdump__std__list(d, value)
 
+
 def qdump__std____1__list(d, value):
     if value.type.size() == 3 * d.ptrSize():
         # C++11 only.
@@ -259,8 +290,10 @@ def qdump__std____1__list(d, value):
                 (prev, p, val) = d.split(typeCode, p)
                 d.putSubItem(i, val)
 
+
 def qform__std__map():
-    return mapForms()
+    return [DisplayFormat.CompactMap]
+
 
 def qdump__std__map(d, value):
     if d.isQnxTarget() or d.isMsvcTarget():
@@ -268,8 +301,9 @@ def qdump__std__map(d, value):
         return
 
     # stuff is actually (color, pad) with 'I@', but we can save cycles/
-    (compare, stuff, parent, left, right, size) = value.split('pppppp')
-    d.check(0 <= size and size <= 100*1000*1000)
+    (compare, stuff, parent, left, right) = value.split('ppppp')
+    size = value["_M_t"]["_M_impl"]["_M_node_count"].integer()
+    d.check(0 <= size and size <= 100 * 1000 * 1000)
     d.putItemCount(size)
 
     if d.isExpanded():
@@ -298,16 +332,23 @@ def qdump__std__map(d, value):
                             break
                         node = node["_M_left"]
 
+
 def qdump_std__map__helper(d, value):
-    (proxy, head, size) = value.split("ppp")
-    d.check(0 <= size and size <= 100*1000*1000)
+    try:
+        _ = value["_Mypair"]["_Myval2"]["_Myval2"]["_Myproxy"]
+        (proxy, head, size) = value.split("ppp")
+    except Exception:
+        (head, size) = value.split("pp")
+    d.check(0 <= size and size <= 100 * 1000 * 1000)
     d.putItemCount(size)
     if d.isExpanded():
         keyType = value.type[0]
         valueType = value.type[1]
         pairType = value.type[3][0]
+
         def helper(node):
-            (left, parent, right, color, isnil, pad, pair) = d.split("pppcc@{%s}" % (pairType.name), node)
+            (left, parent, right, color, isnil, pad, pair) = d.split(
+                "pppcc@{%s}" % (pairType.name), node)
             if left != head:
                 for res in helper(left):
                     yield res
@@ -321,24 +362,31 @@ def qdump_std__map__helper(d, value):
             for (pair, i) in zip(helper(root), d.childRange()):
                 d.putPairItem(i, pair)
 
+
 def qdump__std____debug__map(d, value):
     qdump__std__map(d, value)
 
+
 def qdump__std____debug__set(d, value):
     qdump__std__set(d, value)
 
+
 def qdump__std__multiset(d, value):
     qdump__std__set(d, value)
 
+
 def qdump__std____cxx1998__map(d, value):
     qdump__std__map(d, value)
 
+
 def qform__std__multimap():
-    return mapForms()
+    return [DisplayFormat.CompactMap]
+
 
 def qdump__std__multimap(d, value):
     return qdump__std__map(d, value)
 
+
 def qdumpHelper__std__tree__iterator(d, value, isSet=False):
     treeTypeName = None
     if value.type.name.endswith("::iterator"):
@@ -349,7 +397,7 @@ def qdumpHelper__std__tree__iterator(d, value, isSet=False):
     keyType = treeType[0]
     valueType = treeType[1]
     node = value["_M_node"].dereference()   # std::_Rb_tree_node_base
-    d.putNumChild(1)
+    d.putExpandable()
     d.putEmptyValue()
     if d.isExpanded():
         with Children(d):
@@ -363,7 +411,7 @@ def qdumpHelper__std__tree__iterator(d, value, isSet=False):
                 d.putSubItem("first", key)
                 d.putSubItem("second", value)
             with SubItem(d, "[node]"):
-                d.putNumChild(1)
+                d.putExpandable()
                 d.putEmptyValue()
                 d.putType(" ")
                 if d.isExpanded():
@@ -374,32 +422,41 @@ def qdumpHelper__std__tree__iterator(d, value, isSet=False):
                         d.putSubItem("right", d.createValue(right, nodeType))
                         d.putSubItem("parent", d.createValue(parent, nodeType))
 
+
 def qdump__std___Rb_tree_iterator(d, value):
     qdumpHelper__std__tree__iterator(d, value)
 
+
 def qdump__std___Rb_tree_const_iterator(d, value):
     qdumpHelper__std__tree__iterator(d, value)
 
+
 def qdump__std__map__iterator(d, value):
     qdumpHelper__std__tree__iterator(d, value)
 
+
 def qdump____gnu_debug___Safe_iterator(d, value):
     d.putItem(value["_M_current"])
 
+
 def qdump__std__map__const_iterator(d, value):
     qdumpHelper__std__tree__iterator(d, value)
 
+
 def qdump__std__set__iterator(d, value):
     qdumpHelper__std__tree__iterator(d, value, True)
 
+
 def qdump__std__set__const_iterator(d, value):
     qdumpHelper__std__tree__iterator(d, value, True)
 
+
 def qdump__std____cxx1998__set(d, value):
     qdump__std__set(d, value)
 
+
 def qdumpHelper__std__tree__iterator_MSVC(d, value):
-    d.putNumChild(1)
+    d.putExpandable()
     d.putEmptyValue()
     if d.isExpanded():
         with Children(d):
@@ -415,9 +472,11 @@ def qdumpHelper__std__tree__iterator_MSVC(d, value):
             else:
                 d.putSubItem("value", child)
 
+
 def qdump__std___Tree_const_iterator(d, value):
     qdumpHelper__std__tree__iterator_MSVC(d, value)
 
+
 def qdump__std___Tree_iterator(d, value):
     qdumpHelper__std__tree__iterator_MSVC(d, value)
 
@@ -429,7 +488,7 @@ def qdump__std__set(d, value):
 
     impl = value["_M_t"]["_M_impl"]
     size = impl["_M_node_count"].integer()
-    d.check(0 <= size and size <= 100*1000*1000)
+    d.check(0 <= size and size <= 100 * 1000 * 1000)
     d.putItemCount(size)
     if d.isExpanded():
         valueType = value.type[0]
@@ -452,14 +511,21 @@ def qdump__std__set(d, value):
                     while node["_M_left"].pointer() != 0:
                         node = node["_M_left"]
 
+
 def qdump__std__set__QNX(d, value):
-    (proxy, head, size) = value.split("ppp")
-    d.check(0 <= size and size <= 100*1000*1000)
+    try:
+        _ = value["_Mypair"]["_Myval2"]["_Myval2"]["_Myproxy"]
+        (proxy, head, size) = value.split("ppp")
+    except Exception:
+        (head, size) = value.split("pp")
+    d.check(0 <= size and size <= 100 * 1000 * 1000)
     d.putItemCount(size)
     if d.isExpanded():
-        childType=value.type[0]
+        childType = value.type[0]
+
         def helper(node):
-            (left, parent, right, color, isnil, pad, value) = d.split("pppcc@{%s}" % childType.name, node)
+            (left, parent, right, color, isnil, pad, value) = d.split(
+                "pppcc@{%s}" % childType.name, node)
             if left != head:
                 for res in helper(left):
                     yield res
@@ -473,6 +539,7 @@ def qdump__std__set__QNX(d, value):
             for (item, i) in zip(helper(root), d.childRange()):
                 d.putSubItem(i, item)
 
+
 def std1TreeMin(d, node):
     #_NodePtr __tree_min(_NodePtr __x):
     #    while (__x->__left_ != nullptr)
@@ -484,6 +551,7 @@ def std1TreeMin(d, node):
         node = left
     return node
 
+
 def std1TreeIsLeftChild(d, node):
     # bool __tree_is_left_child(_NodePtr __x):
     #     return __x == __x->__parent_->__left_;
@@ -507,10 +575,11 @@ def std1TreeNext(d, node):
         node = node['__parent_']
     return node['__parent_']
 
+
 def qdump__std____1__set(d, value):
     (proxy, head, size) = value.split("ppp")
 
-    d.check(0 <= size and size <= 100*1000*1000)
+    d.check(0 <= size and size <= 100 * 1000 * 1000)
     d.putItemCount(size)
 
     if d.isExpanded():
@@ -533,21 +602,24 @@ def qdump__std____1__set(d, value):
             for (i, data) in zip(d.childRange(), in_order_traversal(head)):
                 d.putSubItem(i, data)
 
+
 def qdump__std____1__multiset(d, value):
     qdump__std____1__set(d, value)
 
+
 def qform__std____1__map():
-    return mapForms()
+    return [DisplayFormat.CompactMap]
+
 
 def qdump__std____1__map(d, value):
     try:
         (proxy, head, size) = value.split("ppp")
-        d.check(0 <= size and size <= 100*1000*1000)
+        d.check(0 <= size and size <= 100 * 1000 * 1000)
 
     # Sometimes there is extra data at the front. Don't know why at the moment.
     except RuntimeError:
         (junk, proxy, head, size) = value.split("pppp")
-        d.check(0 <= size and size <= 100*1000*1000)
+        d.check(0 <= size and size <= 100 * 1000 * 1000)
 
     d.putItemCount(size)
 
@@ -573,12 +645,15 @@ def qdump__std____1__map(d, value):
             for (i, pair) in zip(d.childRange(), in_order_traversal(head)):
                 d.putPairItem(i, pair, 'key', 'value')
 
+
 def qform__std____1__multimap():
-    return mapForms()
+    return [DisplayFormat.CompactMap]
+
 
 def qdump__std____1__multimap(d, value):
     qdump__std____1__map(d, value)
 
+
 def qdump__std____1__map__iterator(d, value):
     d.putEmptyValue()
     if d.isExpanded():
@@ -587,12 +662,14 @@ def qdump__std____1__map__iterator(d, value):
             d.putSubItem('first', node['first'])
             d.putSubItem('second', node['second'])
 
+
 def qdump__std____1__map__const_iterator(d, value):
     qdump__std____1__map__iterator(d, value)
 
+
 def qdump__std____1__set__iterator(d, value):
     d.putEmptyValue()
-    d.putNumChild(1)
+    d.putExpandable()
     if value.type.name.endswith("::iterator"):
         treeTypeName = value.type.name[:-len("::iterator")]
     elif value.type.name.endswith("::const_iterator"):
@@ -605,27 +682,34 @@ def qdump__std____1__set__iterator(d, value):
             node = node.cast(keyType)
             d.putSubItem('value', node)
 
+
 def qdump__std____1__set_const_iterator(d, value):
     qdump__std____1__set__iterator(d, value)
 
+
 def qdump__std__stack(d, value):
     d.putItem(value["c"])
     d.putBetterType(value.type)
 
+
 def qdump__std____debug__stack(d, value):
     qdump__std__stack(d, value)
 
+
 def qdump__std____1__stack(d, value):
     d.putItem(value["c"])
     d.putBetterType(value.type)
 
+
 def qform__std__string():
-    return [Latin1StringFormat, SeparateLatin1StringFormat,
-            Utf8StringFormat, SeparateUtf8StringFormat ]
+    return [DisplayFormat.Latin1String, DisplayFormat.SeparateLatin1String,
+            DisplayFormat.Utf8String, DisplayFormat.SeparateUtf8String]
+
 
 def qdump__std__string(d, value):
     qdumpHelper_std__string(d, value, d.createType("char"), d.currentItemFormat())
 
+
 def qdumpHelper_std__string(d, value, charType, format):
     if d.isQnxTarget():
         qdumpHelper__std__string__QNX(d, value, charType, format)
@@ -634,40 +718,64 @@ def qdumpHelper_std__string(d, value, charType, format):
         qdumpHelper__std__string__MSVC(d, value, charType, format)
         return
 
-    data = value.extractPointer()
+    # GCC 9, QTCREATORBUG-22753
+    try:
+        data = value["_M_dataplus"]["_M_p"].pointer()
+    except:
+        data = value.extractPointer()
+    try:
+        size = int(value["_M_string_length"])
+        d.putCharArrayHelper(data, size, charType, format)
+        return
+    except:
+        pass
+
     # We can't lookup the std::string::_Rep type without crashing LLDB,
     # so hard-code assumption on member position
     # struct { size_type _M_length, size_type _M_capacity, int _M_refcount; }
-    (size, alloc, refcount) = d.split("ppp", data - 3 * d.ptrSize())
-    refcount = refcount & 0xffffffff
-    d.check(refcount >= -1) # Can be -1 according to docs.
-    d.check(0 <= size and size <= alloc and alloc <= 100*1000*1000)
+    (size, alloc, refcount) = d.split("ppi", data - 3 * d.ptrSize())
+    d.check(refcount >= -1)  # Can be -1 according to docs.
+    d.check(0 <= size and size <= alloc and alloc <= 100 * 1000 * 1000)
     d.putCharArrayHelper(data, size, charType, format)
 
+
 def qdumpHelper__std__string__QNX(d, value, charType, format):
     size = value['_Mysize']
     alloc = value['_Myres']
     _BUF_SIZE = int(16 / charType.size())
-    if _BUF_SIZE <= alloc: #(_BUF_SIZE <= _Myres ? _Bx._Ptr : _Bx._Buf);
+    if _BUF_SIZE <= alloc:  # (_BUF_SIZE <= _Myres ? _Bx._Ptr : _Bx._Buf);
         data = value['_Bx']['_Ptr']
     else:
         data = value['_Bx']['_Buf']
     sizePtr = data.cast(d.charType().pointer())
     refcount = int(sizePtr[-1])
-    d.check(refcount >= -1) # Can be -1 accoring to docs.
-    d.check(0 <= size and size <= alloc and alloc <= 100*1000*1000)
+    d.check(refcount >= -1)  # Can be -1 accoring to docs.
+    d.check(0 <= size and size <= alloc and alloc <= 100 * 1000 * 1000)
     d.putCharArrayHelper(sizePtr, size, charType, format)
 
+
 def qdumpHelper__std__string__MSVC(d, value, charType, format):
-    (proxy, buffer, size, alloc) = value.split("p16spp");
-    _BUF_SIZE = int(16 / charType.size());
-    d.check(0 <= size and size <= alloc and alloc <= 100*1000*1000)
+    try:
+        _ = value["_Mypair"]["_Myval2"]["_Myproxy"]
+        (proxy, buffer, size, alloc) = value.split("p16spp")
+    except Exception:
+        proxy = None
+        (buffer, size, alloc) = value.split("16spp")
+    d.check(0 <= size and size <= alloc and alloc <= 100 * 1000 * 1000)
+    _BUF_SIZE = int(16 / charType.size())
     if _BUF_SIZE <= alloc:
-        (proxy, data) = value.split("pp");
+        if proxy is None:
+            data = value.extractPointer()
+        else:
+            (proxy, data) = value.split("pp")
     else:
-        data = value.address() + d.ptrSize()
+        if proxy is None:
+            data = value.address()
+        else:
+            data = value.address() + d.ptrSize()
     d.putCharArrayHelper(data, size, charType, format)
 
+
 def qdump__std____1__string(d, value):
     firstByte = value.split('b')[0]
     if int(firstByte & 1) == 0:
@@ -697,9 +805,11 @@ def qdump__std____1__wstring(d, value):
 def qdump__std____weak_ptr(d, value):
     return qdump__std__shared_ptr(d, value)
 
+
 def qdump__std__weak_ptr(d, value):
     return qdump__std__shared_ptr(d, value)
 
+
 def qdump__std____1__weak_ptr(d, value):
     return qdump__std____1__shared_ptr(d, value)
 
@@ -712,29 +822,32 @@ def qdump__std__shared_ptr(d, value):
 
     if i.pointer() == 0:
         d.putValue("(null)")
-        d.putNumChild(0)
     else:
         d.putItem(i.dereference())
         d.putBetterType(value.type)
 
+
 def qdump__std____1__shared_ptr(d, value):
     i = value["__ptr_"]
     if i.pointer() == 0:
         d.putValue("(null)")
-        d.putNumChild(0)
     else:
         d.putItem(i.dereference())
         d.putBetterType(value.type)
 
+
 def qdump__std__unique_ptr(d, value):
-    p = d.extractPointer(value)
+    if value.type.size() == d.ptrSize():
+        p = d.extractPointer(value)
+    else:
+        _, p = value.split("pp"); # For custom deleters.
     if p == 0:
         d.putValue("(null)")
-        d.putNumChild(0)
     else:
         d.putItem(d.createValue(p, value.type[0]))
         d.putBetterType(value.type)
 
+
 def qdump__std____1__unique_ptr(d, value):
     qdump__std__unique_ptr(d, value)
 
@@ -745,57 +858,75 @@ def qdump__std__pair(d, value):
     with Children(d):
         key = d.putSubItem('first', first)
         value = d.putSubItem('second', second)
-    d.putField('key', key.value)
-    if key.encoding is not None:
-        d.putField('keyencoded', key.encoding)
-    d.putValue(value.value, value.encoding)
+    key = key.value if key.encoding is None else "..."
+    value = value.value if value.encoding is None else "..."
+    d.putValue('(%s, %s)' % (key, value))
+
 
 def qform__std__unordered_map():
-    return mapForms()
+    return [DisplayFormat.CompactMap]
+
 
 def qform__std____debug__unordered_map():
-    return mapForms()
+    return [DisplayFormat.CompactMap]
+
 
 def qdump__std__unordered_map(d, value):
-    if d.isQnxTarget() or d.isMsvcTarget():
+    if d.isQnxTarget():
         qdump__std__list__QNX(d, value["_List"])
         return
 
-    try:
-        # gcc ~= 4.7
-        size = value["_M_element_count"].integer()
-        start = value["_M_before_begin"]["_M_nxt"]
-    except:
+    if d.isMsvcTarget():
+        _list = value["_List"]
         try:
-            # libc++ (Mac)
-            size = value["_M_h"]["_M_element_count"].integer()
-            start = value["_M_h"]["_M_bbegin"]["_M_node"]["_M_nxt"]
+            _ = _list["_Mypair"]["_Myval2"]["_Myproxy"]
+            (_, start, size) = _list.split("ppp")
+        except Exception:
+            (start, size) = _list.split("pp")
+    else:
+        try:
+            # gcc ~= 4.7
+            size = value["_M_element_count"].integer()
+            start = value["_M_before_begin"]["_M_nxt"]
         except:
             try:
-                # gcc 4.9.1
+                # libc++ (Mac)
                 size = value["_M_h"]["_M_element_count"].integer()
-                start = value["_M_h"]["_M_before_begin"]["_M_nxt"]
+                start = value["_M_h"]["_M_bbegin"]["_M_node"]["_M_nxt"]
             except:
-                # gcc 4.6.2
-                size = value["_M_element_count"].integer()
-                start = value["_M_buckets"].dereference()
-                # FIXME: Pointer-aligned?
-                d.putItemCount(size)
-                # We don't know where the data is
-                d.putNumChild(0)
-                return
+                try:
+                    # gcc 4.9.1
+                    size = value["_M_h"]["_M_element_count"].integer()
+                    start = value["_M_h"]["_M_before_begin"]["_M_nxt"]
+                except:
+                    # gcc 4.6.2
+                    size = value["_M_element_count"].integer()
+                    start = value["_M_buckets"].dereference()
+                    # FIXME: Pointer-aligned?
+                    d.putItemCount(size)
+                    # We don't know where the data is
+                    d.putNumChild(0)
+                    return
 
     d.putItemCount(size)
     if d.isExpanded():
         keyType = value.type[0]
         valueType = value.type[1]
-        typeCode = 'p@{%s}@{%s}' % (value.type[0].name, value.type[1].name)
-        p = start.pointer()
+        if d.isMsvcTarget():
+            typeCode = 'pp@{%s}@{%s}' % (keyType.name, valueType.name)
+            p = d.extractPointer(start)
+        else:
+            typeCode = 'p@{%s}@{%s}' % (keyType.name, valueType.name)
+            p = start.pointer()
         with Children(d, size):
             for i in d.childRange():
-                p, pad, key, pad, val = d.split(typeCode, p)
+                if d.isMsvcTarget():
+                    p, _, _, key, _, val = d.split(typeCode, p)
+                else:
+                    p, _, key, _, val = d.split(typeCode, p)
                 d.putPairItem(i, (key, val))
 
+
 def qdump__std____debug__unordered_map(d, value):
     qdump__std__unordered_map(d, value)
 
@@ -803,12 +934,15 @@ def qdump__std____debug__unordered_map(d, value):
 def qform__std__unordered_multimap():
     return qform__std__unordered_map()
 
+
 def qform__std____debug__unordered_multimap():
     return qform__std____debug__unordered_map()
 
+
 def qdump__std__unordered_multimap(d, value):
     qdump__std__unordered_map(d, value)
 
+
 def qdump__std____debug__unordered_multimap(d, value):
     qdump__std__unordered_multimap(d, value)
 
@@ -851,8 +985,10 @@ def qdump__std__unordered_set(d, value):
                 d.putSubItem(i, d.createValue(p + ptrSize - offset, valueType))
                 p = d.extractPointer(p + offset)
 
+
 def qform__std____1__unordered_map():
-    return mapForms()
+    return [DisplayFormat.CompactMap]
+
 
 def qdump__std____1__unordered_map(d, value):
     (size, _) = value["__table_"]["__p2_"].split("pp")
@@ -899,15 +1035,18 @@ def qdump__std____1__unordered_set(d, value):
 def qdump__std____debug__unordered_set(d, value):
     qdump__std__unordered_set(d, value)
 
+
 def qdump__std__unordered_multiset(d, value):
     qdump__std__unordered_set(d, value)
 
+
 def qdump__std____debug__unordered_multiset(d, value):
     qdump__std__unordered_multiset(d, value)
 
 
 def qform__std__valarray():
-    return arrayForms()
+    return [DisplayFormat.ArrayPlot]
+
 
 def qdump__std__valarray(d, value):
     if d.isMsvcTarget():
@@ -919,7 +1058,8 @@ def qdump__std__valarray(d, value):
 
 
 def qform__std____1__valarray():
-    return arrayForms()
+    return [DisplayFormat.ArrayPlot]
+
 
 def qdump__std____1__valarray(d, value):
     innerType = value.type[0]
@@ -928,9 +1068,20 @@ def qdump__std____1__valarray(d, value):
     d.putItemCount(size)
     d.putPlotData(begin, size, innerType)
 
+def qdump__std__variant(d, value):
+    which = int(value["_M_index"])
+    type = d.templateArgument(value.type, which)
+    d.putValue("<%s:%s>" % (which, type.name))
+
+    d.putNumChild(1)
+    if d.isExpanded():
+        storage = value["_M_u"]["_M_first"]["_M_storage"]
+        with Children(d, 1):
+            d.putSubItem("value", storage.cast(type))
 
 def qform__std__vector():
-    return arrayForms()
+    return [DisplayFormat.ArrayPlot]
+
 
 def qedit__std__vector(d, value, data):
     import gdb
@@ -944,96 +1095,104 @@ def qedit__std__vector(d, value, data):
     cmd = "set (%s[%d])*$d={%s}" % (innerType, n, data)
     gdb.execute(cmd)
 
+
 def qdump__std__vector(d, value):
     if d.isQnxTarget() or d.isMsvcTarget():
-        qdumpHelper__std__vector__QNX(d, value)
+        qdumpHelper__std__vector__msvc(d, value)
+    elif value.hasMember("_M_impl"):
+        qdumpHelper__std__vector__libstdcxx(d, value)
+    elif value.hasMember("__begin_"):
+        qdumpHelper__std__vector__libcxx(d, value)
     else:
-        qdumpHelper__std__vector(d, value, False)
+        qdumpHelper__std__vector__libstdcxx(d, value)
 
-def qdumpHelper__std__vector(d, value, isLibCpp):
-    innerType = value.type[0]
-    isBool = innerType.name == 'bool'
 
-    if isBool:
-        if isLibCpp:
-            start = value["__begin_"].pointer()
-            size = value["__size_"]
-            alloc = size
-        else:
-            start = value["_M_start"]["_M_p"].pointer()
-            soffset = value["_M_start"]["_M_offset"].integer()
-            finish = value["_M_finish"]["_M_p"].pointer()
-            foffset = value["_M_finish"]["_M_offset"].integer()
-            alloc = value["_M_end_of_storage"].pointer()
-            size = (finish - start) * 8 + foffset - soffset # 8 is CHAR_BIT.
-    else:
-        if isLibCpp:
-            start = value["__begin_"].pointer()
-            finish = value["__end_"].pointer()
-            alloc = value["__end_cap_"].pointer()
-        else:
-            start = value["_M_start"].pointer()
-            finish = value["_M_finish"].pointer()
-            alloc = value["_M_end_of_storage"].pointer()
-        size = int((finish - start) / innerType.size())
-        d.check(finish <= alloc)
-        if size > 0:
-            d.checkPointer(start)
-            d.checkPointer(finish)
-            d.checkPointer(alloc)
-
-    d.check(0 <= size and size <= 1000 * 1000 * 1000)
-
-    d.putItemCount(size)
-    if isBool:
-        if d.isExpanded():
-            with Children(d, size, maxNumChild=10000, childType=innerType):
-                for i in d.childRange():
-                    q = start + int(i / 8)
-                    with SubItem(d, i):
-                        d.putValue((int(d.extractPointer(q)) >> (i % 8)) & 1)
-                        d.putType("bool")
-                        d.putNumChild(0)
-    else:
-        d.putPlotData(start, size, innerType)
-
-def qdumpHelper__std__vector__QNX(d, value):
-    innerType = value.type[0]
-    isBool = innerType.name == 'bool'
-    if isBool:
-        (proxy1, proxy2, start, last, end, size) = value.split("pppppi")
-    else:
-        (proxy, start, last, end) = value.split("pppp")
-        size = (last - start) // innerType.size()
-
-    d.check(0 <= size and size <= 1000 * 1000 * 1000)
-    d.check(last <= end)
+def qdumpHelper__std__vector__nonbool(d, start, finish, alloc, inner_type):
+    size = int((finish - start) / inner_type.size())
+    d.check(finish <= alloc)
     if size > 0:
         d.checkPointer(start)
-        d.checkPointer(last)
-        d.checkPointer(end)
+        d.checkPointer(finish)
+        d.checkPointer(alloc)
+    d.check(0 <= size and size <= 1000 * 1000 * 1000)
+    d.putItemCount(size)
+    d.putPlotData(start, size, inner_type)
 
+
+def qdumpHelper__std__vector__bool(d, start, size, inner_type):
+    d.check(0 <= size and size <= 1000 * 1000 * 1000)
     d.putItemCount(size)
     if d.isExpanded():
-        if isBool:
-            with Children(d, size, maxNumChild=10000, childType=innerType):
-                for i in d.childRange():
-                    q = start + int(i / 8)
-                    with SubItem(d, i):
-                        d.putValue((d.extractPointer(q) >> (i % 8)) & 1)
-                        d.putType("bool")
-                        d.putNumChild(0)
-        else:
-            d.putPlotData(start, size, innerType)
+        with Children(d, size, maxNumChild=10000, childType=inner_type):
+            for i in d.childRange():
+                q = start + int(i / 8)
+                with SubItem(d, i):
+                    d.putValue((int(d.extractPointer(q)) >> (i % 8)) & 1)
+                    d.putType("bool")
+
+
+def qdumpHelper__std__vector__libstdcxx(d, value):
+    inner_type = value.type[0]
+    if inner_type.name == "bool":
+        start = value["_M_start"]["_M_p"].pointer()
+        soffset = value["_M_start"]["_M_offset"].integer()
+        finish = value["_M_finish"]["_M_p"].pointer()
+        foffset = value["_M_finish"]["_M_offset"].integer()
+        alloc = value["_M_end_of_storage"].pointer()
+        size = (finish - start) * 8 + foffset - soffset  # 8 is CHAR_BIT.
+        qdumpHelper__std__vector__bool(d, start, size, inner_type)
+    else:
+        start = value["_M_start"].pointer()
+        finish = value["_M_finish"].pointer()
+        alloc = value["_M_end_of_storage"].pointer()
+        qdumpHelper__std__vector__nonbool(d, start, finish, alloc, inner_type)
+
+
+def qdumpHelper__std__vector__libcxx(d, value):
+    inner_type = value.type[0]
+    if inner_type.name == "bool":
+        start = value["__begin_"].pointer()
+        size = value["__size_"].integer()
+        qdumpHelper__std__vector__bool(d, start, size, inner_type)
+    else:
+        start = value["__begin_"].pointer()
+        finish = value["__end_"].pointer()
+        alloc = value["__end_cap_"].pointer()
+        qdumpHelper__std__vector__nonbool(d, start, finish, alloc, inner_type)
+
+
+def qdumpHelper__std__vector__msvc(d, value):
+    inner_type = value.type[0]
+    if inner_type.name == "bool":
+        try:
+            _ = value["_Myproxy"]
+            proxy1, proxy2, start, finish, alloc, size = value.split("pppppi")
+        except RuntimeError:
+            start, finish, alloc, size = value.split("pppi")
+        d.check(0 <= size and size <= 1000 * 1000 * 1000)
+        qdumpHelper__std__vector__bool(d, start, size, inner_type)
+    else:
+        try:
+            _ = value["_Mypair"]["_Myval2"]["_Myproxy"]
+            proxy, start, finish, alloc = value.split("pppp")
+        except RuntimeError:
+            start, finish, alloc = value.split("ppp")
+        size = (finish - start) // inner_type.size()
+        d.check(0 <= size and size <= 1000 * 1000 * 1000)
+        qdumpHelper__std__vector__nonbool(d, start, finish, alloc, inner_type)
+
 
 def qform__std____1__vector():
-    return arrayForms()
+    return [DisplayFormat.ArrayPlot]
+
 
 def qdump__std____1__vector(d, value):
-    qdumpHelper__std__vector(d, value, True)
+    qdumpHelper__std__vector__libcxx(d, value)
+
 
 def qform__std____debug__vector():
-    return arrayForms()
+    return [DisplayFormat.ArrayPlot]
+
 
 def qdump__std____debug__vector(d, value):
     qdump__std__vector(d, value)
@@ -1057,37 +1216,48 @@ def qdump__std__initializer_list(d, value):
     if d.isExpanded():
         d.putPlotData(start, size, innerType)
 
+
 def qedit__std__string(d, value, data):
     d.call('void', value, 'assign', '"%s"' % data.replace('"', '\\"'))
 
+
 def qedit__string(d, expr, value):
     qedit__std__string(d, expr, value)
 
+
 def qedit__std____cxx11__string(d, expr, value):
     qedit__std__string(d, expr, value)
 
+
 def qedit__std__wstring(d, value, data):
     d.call('void', value, 'assign', 'L"%s"' % data.replace('"', '\\"'))
 
+
 def qedit__wstring(d, expr, value):
     qedit__std__wstring(d, expr, value)
 
+
 def qedit__std____cxx11__wstring(d, expr, value):
     qedit__std__wstring(d, expr, value)
 
+
 def qdump__string(d, value):
     qdump__std__string(d, value)
 
+
 def qform__std__wstring():
-    return [SimpleFormat, SeparateFormat]
+    return [DisplayFormat.Simple, DisplayFormat.Separate]
+
 
 def qdump__std__wstring(d, value):
     qdumpHelper_std__string(d, value, d.createType('wchar_t'), d.currentItemFormat())
 
+
 def qdump__std__basic_string(d, value):
     innerType = value.type[0]
     qdumpHelper_std__string(d, value, innerType, d.currentItemFormat())
 
+
 def qdump__std____cxx11__basic_string(d, value):
     innerType = value.type[0]
     try:
@@ -1097,24 +1267,30 @@ def qdump__std____cxx11__basic_string(d, value):
         d.putEmptyValue()
         d.putPlainChildren(value)
         return
-    d.check(0 <= size) #and size <= alloc and alloc <= 100*1000*1000)
+    d.check(0 <= size)  # and size <= alloc and alloc <= 100*1000*1000)
     d.putCharArrayHelper(data, size, innerType, d.currentItemFormat())
 
+
 def qform__std____cxx11__string(d, value):
     qform__std__string(d, value)
 
+
 def qdump__std____cxx11__string(d, value):
     (data, size) = value.split("pI")
-    d.check(0 <= size) #and size <= alloc and alloc <= 100*1000*1000)
+    d.check(0 <= size)  # and size <= alloc and alloc <= 100*1000*1000)
     d.putCharArrayHelper(data, size, d.charType(), d.currentItemFormat())
 
 # Needed only to trigger the form report above.
+
+
 def qform__std____cxx11__string():
     return qform__std__string()
 
+
 def qform__std____cxx11__wstring():
     return qform__std__wstring()
 
+
 def qdump__std____1__basic_string(d, value):
     innerType = value.type[0].name
     if innerType == "char":
@@ -1122,14 +1298,17 @@ def qdump__std____1__basic_string(d, value):
     elif innerType == "wchar_t":
         qdump__std____1__wstring(d, value)
     else:
-        warn("UNKNOWN INNER TYPE %s" % innerType)
+        d.warn("UNKNOWN INNER TYPE %s" % innerType)
+
 
 def qdump__wstring(d, value):
     qdump__std__wstring(d, value)
 
+
 def qdump__std____1__once_flag(d, value):
     qdump__std__once_flag(d, value)
 
+
 def qdump__std__once_flag(d, value):
     d.putValue(value.split("i")[0])
     d.putBetterType(value.type)
@@ -1150,7 +1329,7 @@ def qdump____gnu_cxx__hash_set(d, value):
             bucketFinish = buckets["_M_finish"]
             p = bucketStart
             itemCount = 0
-            for i in xrange((bucketFinish.pointer() - bucketStart.pointer()) // d.ptrSize()):
+            for i in range((bucketFinish.pointer() - bucketStart.pointer()) // d.ptrSize()):
                 if p.dereference().pointer():
                     cur = p.dereference()
                     while cur.pointer():
@@ -1161,26 +1340,26 @@ def qdump____gnu_cxx__hash_set(d, value):
 
 
 def qdump__uint8_t(d, value):
-    d.putNumChild(0)
     d.putValue(value.integer())
 
+
 def qdump__int8_t(d, value):
-    d.putNumChild(0)
     d.putValue(value.integer())
 
+
 def qdump__std__byte(d, value):
-    d.putNumChild(0)
     d.putValue(value.integer())
 
+
 def qdump__std__optional(d, value):
     innerType = value.type[0]
-    (initialized, pad, payload) = d.split('b@{%s}' % innerType.name, value)
+    (payload, pad, initialized) = d.split('{%s}@b' % innerType.name, value)
     if initialized:
         d.putItem(payload)
         d.putBetterType(value.type)
     else:
         d.putSpecialValue("uninitialized")
-        d.putNumChild(0)
+
 
 def qdump__std__experimental__optional(d, value):
     qdump__std__optional(d, value)
